@@ -1,3 +1,4 @@
+# coding=utf8
 import copy
 import datetime as dt
 from decimal import Decimal
@@ -11,7 +12,8 @@ import os
 import os.path
 import socket
 import sys
-from threading import Timer
+import six
+# from threading import Timer
 import mimetypes
 
 import paste.deploy
@@ -26,7 +28,7 @@ import aalam_common as zc
 import aalam_common.sqa as zsqa
 from aalam_common.debug import logging
 import aalam_common.exceptions as zexc
-from aalam_common.utils import datetime, date
+from aalam_common.utils import datetime, date, report_bug
 from aalam_common import debug
 from aalam_common.config import cfg
 
@@ -41,6 +43,34 @@ class AALAMRequest(webob.Request):
     def __repr__(self):
         return "AALAMRequest at %x" % self
 
+    def _filterout_system_params(self):
+        if '_is_nohook' not in self.environ['webob.adhoc_attrs']:
+            _g = self.GET
+            _p = self.POST
+            self.environ['webob.adhoc_attrs']['_is_nohook'] = False
+            if 'nohook' in _g:
+               self.environ['webob.adhoc_attrs']['_is_nohook'] = True
+            if 'nohook' in _p:
+               self.environ['webob.adhoc_attrs']['_is_nohook'] = True
+            _g = {k: v for k, v in _g.items() if k != 'nohook'}
+            _p = {k: v for k, v in _p.items() if k != 'nohook'}
+            self.params = webob.multidict.NestedMultiDict(_g, _p)
+
+    @property
+    def is_nohook(self):
+        self._filterout_system_params()
+        return self.environ['webob.adhoc_attrs'].get('_is_nohook', False)
+
+    @property
+    def params(self):
+        if self._params is None:
+            self._filterout_system_params()
+        return self._params
+
+    @params.setter
+    def params(self, value):
+        self.environ['webob.adhoc_attrs']['_params'] = value
+
     def __getattr__(self, name):
         try:
             return super(AALAMRequest, self).__getattr__(name)
@@ -54,7 +84,7 @@ class AALAMRequest(webob.Request):
                 self.sqa_session = retval = zsqa.Session(database)
                 return retval
             elif name == "sqa_sessions":
-                route_args = str(self.environ['wsgiorg.routing_args'][1])
+                route_args = self.environ['wsgiorg.routing_args'][1]
                 databases = route_args.get("databases", None)
                 if not databases:
                     logging.error("'databases' routing args not present")
@@ -70,7 +100,7 @@ def cleanup():
     try:
         if zc.CALLBACK_CLEANUP in app_callbacks:
             app_callbacks[zc.CALLBACK_CLEANUP]()
-    except:
+    except Exception:
         pass
     logging.warn("Killing the wsgi server")
     wsgi_listener.kill()
@@ -78,7 +108,15 @@ def cleanup():
     def terminate_now():
         sys.exit(100)
 
-    Timer(CLEANUP_WAIT_PERIOD, terminate_now, ()).start()
+    def terminate_delayed():
+        logging.warn("Terminating after a delay of %s seconds"
+                     % CLEANUP_WAIT_PERIOD)
+        eventlet.sleep(CLEANUP_WAIT_PERIOD)
+        logging.warn("Terminator woke up, exiting now")
+        terminate_now()
+
+    eventlet.spawn_n(terminate_delayed)
+    # Timer(CLEANUP_WAIT_PERIOD, terminate_now, ()).start()
 
 
 class WSGIServer(object):
@@ -97,8 +135,8 @@ class WSGIServer(object):
 
     def _write_paste_ini(self, paste_file):
         if not os.path.exists(paste_file) or os.path.getsize(paste_file) == 0:
-            middlewares = ["router", "auth", "sqa_session",
-                           "roles", "statics", "hooks", "system"]
+            middlewares = ["router", "auth", "roles", "statics",
+                           "hooks", "system", "sqa_session"]
             paste_conf = {"router": "aalam_common.wsgi:Router.factory",
                           cfg.CONF.app_code: "aalam_common.wsgi:AppFactory",
                           "auth": "aalam_common.auth:Auth.factory",
@@ -133,7 +171,8 @@ paste.app_factory = %(factory)s
             with open(paste_file, 'w') as f:
                 f.write(paste)
             os.chown(paste_file, os.geteuid(), os.getegid())
-            os.chmod(paste_file, 0700)
+            # Change mode to 0700
+            os.chmod(paste_file, stat.S_IRWXU)
 
     def _load_app_entry(self):
         self.app_entry = None
@@ -222,7 +261,8 @@ paste.app_factory = %(factory)s
         # check if the directory exists, else create it
         if not os.path.exists(dir_name):
             os.makedirs(dir_name)
-            os.chmod(dir_name, 0750)
+            # change mode to 0750
+            os.chmod(dir_name, stat.S_IRWXU | stat.S_IRGRP | stat.S_IXGRP)
             gid = grp.getgrnam("www-data").gr_gid
             os.chown(dir_name, uid, gid)
 
@@ -242,8 +282,9 @@ paste.app_factory = %(factory)s
         if self._state == zc.STATE_MIGRATE:
             if zc.CALLBACK_MIGRATE in app_callbacks:
                 try:
-                    app_callbacks[zc.CALLBACK_MIGRATE](os.environ['MIGRATE_FROM'])
-                except:
+                    app_callbacks[zc.CALLBACK_MIGRATE](
+                        os.environ['MIGRATE_FROM'])
+                except Exception:
                     os.unlink(os.path.join(
                         WSGI_SOCKET_ROOT, cfg.CONF.app_provider_code,
                         cfg.CONF.app_code + MIGRATE_LOCK))
@@ -255,19 +296,26 @@ paste.app_factory = %(factory)s
                 self._run_server, self._app, self._sock)
             try:
                 wsgi_listener.wait()
-            except:
+            except Exception:
                 pass
             logging.warn("WSGI server finished successfully")
-            self._pool.waitall()
+            logging.warn("WSGI server pool num open connections %s"
+                         % self._pool.running())
+            # waitall sometimes never returns
+            # self._pool.waitall()
             logging.warn("Server pool waited for pending connections")
 
     def _run_server(self, app, sock):
         logging.info("Starting the server")
-        log = file("/dev/null", "w+")
-        eventlet.wsgi.server(sock, app, custom_pool=self._pool,
-                             protocol=self.UnixDomainHttpProtocol,
-                             log_output=False, log=log,
-                             debug=True)
+        log = open("/dev/null", "w+")
+        from greenlet import GreenletExit
+        try:
+            eventlet.wsgi.server(sock, app, custom_pool=self._pool,
+                                 protocol=self.UnixDomainHttpProtocol,
+                                 log_output=False, log=log,
+                                 debug=True)
+        except GreenletExit:
+            pass
 
 
 class Middleware(object):
@@ -346,7 +394,7 @@ class BaseHandler(object):
         raise webob.exc.HTTPMethodNotAllowed()
 
     def _json_deserializer(self, request):
-        return json.loads(request.body)
+        return json.loads(request.text)
 
     def json_deserializer(self, request):
         return self._json_deserializer(request)
@@ -357,7 +405,7 @@ class BaseHandler(object):
         if call:
             return call(request)
 
-        if request.content_length is 0:
+        if not request.content_length:
             return {}
 
         if request.content_type == "application/json":
@@ -377,7 +425,7 @@ class BaseHandler(object):
             return date.to_user(obj)
 
     def _json_serializer(self, app_data, response):
-        primitives = (int, str, float, long, bool)
+        primitives = six.integer_types + (str, float, bool)
         if isinstance(app_data, list) or \
            isinstance(app_data, dict):
             body = app_data
@@ -391,7 +439,11 @@ class BaseHandler(object):
                 body = "Unable to serialize %s" % type(app_data)
 
         response.content_type = "application/json"
-        response.body = json.dumps(body, default=self._json_default)
+        out_data = json.dumps(body, default=self._json_default)
+        if six.PY2:
+            response.body = out_data
+        elif six.PY3:
+            response.text = out_data
 
     def json_serializer(self, app_data, response):
         self._json_serializer(app_data, response)
@@ -399,9 +451,13 @@ class BaseHandler(object):
     def _html_serializer(self, data, response):
         response.content_type = "text/html"
 
-        @htmlmin(remove_comments=True, keep_pre=True)
+        @htmlmin(remove_comments=True, keep_pre=True,
+                 remove_optional_attribute_quotes=False)
         def minify():
-            return unicode(data)
+            if six.PY2:
+                return unicode(data)
+            else:
+                return data
 
         response.text = minify()
 
@@ -439,6 +495,16 @@ class BaseHandler(object):
         except Exception as e:
             if getattr(e, "__module__", None) == "webob.exc":
                 raise e
+            try:
+                exc_type, exc_value, exc_tb = sys.exc_info()
+                etb = debug.ExceptionTraceback(exc_type, exc_value, exc_tb)
+                lines = etb.format_exception()
+                eventlet.spawn_n(
+                    report_bug, 'request', exc_type, exc_value, lines,
+                    url="%s %s" % (request.method, request.path))
+            finally:
+                exc_type = exc_value = exc_tb = None
+                etb = None
             import traceback
             traceback.print_exc()
             logging.error("Application failed for this url")
@@ -557,7 +623,7 @@ class StaticsMiddleware(Middleware):
             mime = mime or "application/octet-stream"
             response.content_type = mime
             response.status_code = 200
-            with open(fname if not fname_gz else fname_gz, "r") as fd:
+            with open(fname if not fname_gz else fname_gz, "rb") as fd:
                 response.body = fd.read()
         else:
             response.status_code = 304
